#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <string.h>
#include <unistd.h>
#include "htmlgen.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "image-ops.h"
#include "config_file.h"
#include "filename.h"
#include "upload.h"
#include "config_dialog.h"
#include "preview.h"
#include "dialog.h"

extern struct imgsize is;
extern struct imgsize ts;
extern struct html_data hd;
extern const char * config[];
extern struct mainwin win;


void
on_button_ok_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
	/* Plan:
	    - check config,
	    - show html,
	    - resize image,
	    - create thumbnail,
	    - download,
	    - remove generated files,
	    - exit.
	 */
	GString * imgf;
	GString * tnf;
	const char * filelist[] = {hd.filename_full, NULL, NULL};
	int ust;
	GString * html;

	if (check_config(config) == FALSE)
	{
		/* Force preferences window to appear */
		if (show_config_dialog (GTK_WINDOW (win.window)) == FALSE)
			/* Not all required prefs set */
			return;
	}

	hd.alt = gtk_entry_get_text (GTK_ENTRY (win.entry_img_alt));

	html = get_html_code(&hd);
	if (html == NULL)
		/* User pressed "Cancel" */
		return;

	imgf = g_string_sized_new(128);
	tnf = g_string_sized_new(128);
	
	if (hd.is_resize)
	{
		g_string_printf(imgf, "%s%c%s", gettmpdir(), '/', hd.filename);
		filelist[0] = imgf->str;
		image_resize(hd.filename_full, is.scaled_size_x,
				is.scaled_size_y, imgf->str);
	}
	
	if (hd.is_thumbnailed)
	{
		g_string_printf(tnf, "%s%c%s%s", gettmpdir(), '/',
				config[C_THUMB_PREFIX], hd.filename);
		filelist[1] = tnf->str;
		image_resize(hd.filename_full, ts.scaled_size_x,
				ts.scaled_size_y, tnf->str);
	}

	ust = upload_files(filelist, config);
	
	if (hd.is_resize)
		unlink(imgf->str);
	if (hd.is_thumbnailed)
		unlink(tnf->str);
	
	g_string_free(imgf, TRUE);
	g_string_free(tnf, TRUE);

	if (ust != 0)
		/* upload seems to be failed */
		return;

	/* Print HTML code to stdout */
	g_print(html->str);
	g_print("\n");
	g_string_free(html, TRUE);

	gtk_main_quit();
}


void
on_button_openfile_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	 GtkWidget * fileopen_dialog = create_fileselection1();
	 gtk_widget_show(fileopen_dialog);
}

void on_button_config_clicked (GtkButton * button,
		gpointer user_data)
{
	show_config_dialog (GTK_WINDOW (win.window));
}

gboolean load_image(const char * filename)
{
	const char *img_info;
	int width, height;
	const char * file;

 	if (set_preview_file(filename) < 0)
	{ /* gdk_pixbuf was unable to load image */
		GString *s = g_string_sized_new (128);
		g_string_printf (s, N_("<big><b>Error:</b> file</big>\n"
					"<tt>%s</tt>\n"
					"<big>does not look like "
					"an image file.</big>"),
					filename);
		show_error(_("Error"),_(s->str), NULL);
		g_string_free (s, TRUE);
		set_sensitive(&win, FALSE);
		return FALSE;
	}
	
	img_info = get_image_info(filename);
	
	gtk_label_set_text (GTK_LABEL(win.label_img_info), img_info);
	gtk_label_set_text (GTK_LABEL(win.label_filename), 
			g_filename_to_utf8(filename, -1,
				NULL, NULL, NULL));
	set_sensitive(&win, TRUE);

	get_image_size(&width, &height);
	set_image_size(&is, width, height);
	set_image_size(&ts, width, height);

	hd.filename_full = filename;
	file = getfilename(filename);
	hd.filename = file;
	hd.filename_utf8 =
		strdup(g_filename_to_utf8(file, -1, NULL, NULL, NULL));

	return TRUE;
}

void
store_filename                         (GtkFileSelection *file_selector,
                                        gpointer         user_data)
{
	const gchar *filename;
     
	filename = gtk_file_selection_get_filename (
			GTK_FILE_SELECTION (file_selector));

	if (! load_image(filename))
		return;

	gtk_widget_destroy(GTK_WIDGET(file_selector));
	
}

void
on_checkbutton_scale_toggled           (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gboolean active = gtk_toggle_button_get_active (togglebutton);
	hd.is_resize = active;
	gtk_widget_set_sensitive(is.container, active);
}


void
on_checkbutton_thumb_toggled           (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gboolean active = gtk_toggle_button_get_active (togglebutton);
	hd.is_thumbnailed = active;
	gtk_widget_set_sensitive(ts.container, active);
}

void on_checkbutton_newwin_toggled	(GtkToggleButton *togglebutton,
					gpointer         user_data)
{
	hd.is_in_new_window = gtk_toggle_button_get_active (togglebutton);
}
