/* config_dialog.c
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include "config_file.h"
#include "config_dialog.h"
#include "support.h"
#include "dialog.h"

#define MSS_IN_SECOND 1000.

extern char * config[];
struct config_dialog cd;

char * config_name[C_END] = 
{
	N_("<b>Image URL prefix</b>"),	/* C_URL_PREFIX */
	N_("<b>Thumbnail prefix</b>"),	/* C_THUMB_PREFIX */
	N_("Upload using"),		/* C_UPLOAD_PROTO */
	N_("<b>User name</b>"),		/* C_UPLOAD_USER */
	N_("Password"),			/* C_UPLOAD_PASS */
	N_("<b>Host name</b>"),		/* C_UPLOAD_HOST */
	N_("<b>Directory</b>"),		/* C_UPLOAD_PATH */
	NULL, NULL, NULL, NULL,		/* C_IMG_... */
	NULL, NULL, NULL, NULL,		/* C_TN_... */
	NULL				/* C_PREVIEW_UPD_INT */
};

/* Which options are required? */
gboolean config_required[C_END] =
{
	TRUE,				/* C_URL_PREFIX */
	TRUE,				/* C_THUMB_PREFIX */
	TRUE,				/* C_UPLOAD_PROTO */
	TRUE,				/* C_UPLOAD_USER */
	FALSE,				/* C_UPLOAD_PASS */
	TRUE,				/* C_UPLOAD_HOST */
	TRUE,				/* C_UPLOAD_PATH */
	FALSE, FALSE, FALSE, FALSE,	/* C_IMG_... */
	FALSE, FALSE, FALSE, FALSE,	/* C_TN_... */
	FALSE				/* C_PREVIEW_UPD_INT */
};

const char *itoa(int i)
{
	static char buf[12];
	
	snprintf(buf, sizeof(buf), "%d", i);
	return buf;
}

const char *ftoa(double f)
{
	static char buf[G_ASCII_DTOSTR_BUF_SIZE];

	g_ascii_dtostr (buf, sizeof(buf), f);
	return buf;
}

#define GET_ENTRY(entry) \
	strdup (gtk_entry_get_text (GTK_ENTRY (entry)))

#define GET_SPINBUTTON(entry) \
	strdup (itoa ( gtk_spin_button_get_value_as_int ( \
		GTK_SPIN_BUTTON (entry))))

#define GET_SPINBUTTON_FLOAT(entry, factor) \
	strdup (ftoa ( gtk_spin_button_get_value ( \
		GTK_SPIN_BUTTON (entry)) * factor))

#define GET_ACTIVE(entry) \
	strdup (itoa ( (int)gtk_toggle_button_get_active( \
		GTK_TOGGLE_BUTTON (entry))))

const char * get_img_format (GtkWidget * format)
{
	int sel;

	sel = gtk_option_menu_get_history (GTK_OPTION_MENU (format));
	switch (sel)
	{
		case F_PNG:
			return "png";
			break;
		case F_JPEG:
			return "jpeg";
			break;
		case F_GIF:
			return "gif";
			break;
		default:
			/* this should not happen */
			return "";
	}
}

void get_config_from_dialog (const struct config_dialog * s,
		char * config[])
{
	int sel;
	
	config[C_URL_PREFIX] = GET_ENTRY (s->url_prefix_entry);
	config[C_THUMB_PREFIX] = GET_ENTRY (s->thumb_prefix_entry);
	config[C_PREVIEW_UPD_INT] = GET_SPINBUTTON_FLOAT (
			s->preview_upd_int, MSS_IN_SECOND);

	sel = gtk_option_menu_get_history (GTK_OPTION_MENU (s->upload_proto));
	switch (sel)
	{
		case UP_FTP:
			config[C_UPLOAD_PROTO] = strdup("ftp");
			break;
		case UP_SCP:
			config[C_UPLOAD_PROTO] = strdup("scp");
			break;
		default: /* this should not happen */
			config[C_UPLOAD_PROTO] = NULL;
	}

	config[C_UPLOAD_USER] = GET_ENTRY (s->upload_user);
	config[C_UPLOAD_PASS] = GET_ENTRY (s->upload_pass);
	config[C_UPLOAD_HOST] = GET_ENTRY (s->upload_host);
	config[C_UPLOAD_PATH] = GET_ENTRY (s->upload_path);

	config[C_IMG_WIDTH] = GET_SPINBUTTON (s->image.width);
	config[C_IMG_HEIGHT] = GET_SPINBUTTON (s->image.height);
	config[C_IMG_MAXISABS] = GET_ACTIVE (s->image.max_is_abs);
	config[C_IMG_FORMAT] = strdup (get_img_format (s->image.format));

	config[C_TN_WIDTH] = GET_SPINBUTTON (s->thumb.width);
	config[C_TN_HEIGHT] = GET_SPINBUTTON (s->thumb.height);
	config[C_TN_MAXISABS] = GET_ACTIVE (s->thumb.max_is_abs);
	config[C_TN_FORMAT] = strdup (get_img_format (s->thumb.format));
}

void on_config_save_clicked (GtkButton * button,
		gpointer user_data)
{
	char * new_config[C_END];
	struct config_dialog * d = (struct config_dialog *) user_data;

	get_config_from_dialog(d, new_config);
	if (check_config(new_config) == FALSE)
		return;
	set_config(config, new_config);
	write_config(config);
	d->saved = 1;
	gtk_main_quit();
}

void upload_proto_callback (GtkOptionMenu * menu, gpointer user_data)
{
	gboolean active;
	int sel;
	struct config_dialog * s = (struct config_dialog *) user_data;
	
	sel = gtk_option_menu_get_history (menu);
	active = (sel == UP_SCP)? FALSE: TRUE;

	gtk_widget_set_sensitive (s->upload_pass, active);
	gtk_widget_set_sensitive (s->upload_pass_label, active);
}

#define SET_ENTRY(entry, value) \
	if (value != NULL) \
		gtk_entry_set_text (GTK_ENTRY (entry), value)

#define SET_SPINBUTTON(entry, value) \
	if (value != NULL) \
		gtk_spin_button_set_value (GTK_SPIN_BUTTON (entry), \
			atoi (value))

#define SET_SPINBUTTON_FLOAT(entry, value, factor) \
	if (value != NULL) \
		gtk_spin_button_set_value (GTK_SPIN_BUTTON (entry), \
			atof (value) / factor)

#define SET_ACTIVE(entry, value) \
	if (value != NULL) \
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (entry), \
			value[0] == '1')

void set_img_format(const struct config_dialog_imgopt * s,
		const char * format)
{
	int sel;
	if (format == NULL)
		return;

	if (strcmp(format, "png") == 0)
		sel = F_PNG;
	else if (strcmp(format, "jpeg") == 0)
		sel = F_JPEG;
	else if (strcmp(format, "gif") == 0)
		sel = F_GIF;
	else /* this shouldn't happen */
		sel = F_END;

	gtk_option_menu_set_history (GTK_OPTION_MENU (s->format), sel);
	gtk_menu_item_activate (GTK_MENU_ITEM (s->format_items[sel]));
}

void set_dialog_from_config(const struct config_dialog * s,
		const char * config[])
{
	int sel;
	
	SET_ENTRY (s->url_prefix_entry, config[C_URL_PREFIX]);
	SET_ENTRY (s->thumb_prefix_entry, config[C_THUMB_PREFIX]);
	SET_SPINBUTTON_FLOAT (s->preview_upd_int,
			config[C_PREVIEW_UPD_INT], MSS_IN_SECOND);

	if (config[C_UPLOAD_PROTO] != NULL)
	{
		if (strcmp (config[C_UPLOAD_PROTO], "ftp") == 0)
			sel = UP_FTP;
		else if (strcmp (config[C_UPLOAD_PROTO], "scp") == 0)
			sel = UP_SCP;
		else /* this shouldn't happen */
			sel = UP_END;
		gtk_option_menu_set_history (
			GTK_OPTION_MENU (s->upload_proto), sel);
		gtk_menu_item_activate 
			GTK_MENU_ITEM (s->upload_proto_items[sel]);
	}
	
	SET_ENTRY (s->upload_user, config[C_UPLOAD_USER]);
	SET_ENTRY (s->upload_pass, config[C_UPLOAD_PASS]);
	SET_ENTRY (s->upload_host, config[C_UPLOAD_HOST]);
	SET_ENTRY (s->upload_path, config[C_UPLOAD_PATH]);

	SET_SPINBUTTON(s->image.width, config[C_IMG_WIDTH]);
	SET_SPINBUTTON(s->image.height, config[C_IMG_HEIGHT]);
	SET_ACTIVE(s->image.max_is_abs, config[C_IMG_MAXISABS]);
	set_img_format(&s->image, config[C_IMG_FORMAT]);
	
	SET_SPINBUTTON(s->thumb.width, config[C_TN_WIDTH]);
	SET_SPINBUTTON(s->thumb.height, config[C_TN_HEIGHT]);
	SET_ACTIVE(s->thumb.max_is_abs, config[C_TN_MAXISABS]);
	set_img_format(&s->thumb, config[C_TN_FORMAT]);
}

GtkWidget * create_image_options (const char * str,
		struct config_dialog_imgopt * s)
{
	GtkWidget * frame;
	GtkWidget * frame_label;
	GtkWidget * table;
	GtkWidget * label_width;
	GtkObject * width_adj;
	GtkWidget * label_height;
	GtkObject * height_adj;
	GtkWidget * label_format;
	GtkWidget * item;
	GtkWidget * menu;
	GSList * group = NULL;

	/* Frame */
	frame = gtk_frame_new (NULL);
	gtk_widget_show (frame);
	gtk_container_set_border_width (GTK_CONTAINER (frame), 2);

	frame_label = gtk_label_new (str);
	gtk_widget_show (frame_label);
	gtk_frame_set_label_widget (GTK_FRAME (frame), frame_label);
	gtk_label_set_justify (GTK_LABEL (frame_label), GTK_JUSTIFY_LEFT);

	/* Table */
	table = gtk_table_new (4, 2, FALSE);
	gtk_widget_show (table);
	gtk_container_add (GTK_CONTAINER (frame), table);
	gtk_container_set_border_width (GTK_CONTAINER (table), 0);

	/* Width */
	label_width = gtk_label_new (_("Max width"));
	gtk_widget_show (label_width);
	gtk_table_attach (GTK_TABLE (table), label_width, 0, 1, 0, 1,
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		       	2, 0);
	gtk_label_set_justify (GTK_LABEL (label_width), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label_width), 1, 0.5);

	width_adj = gtk_adjustment_new (0, 0, 10000, 1, 10, 10);
	s->width = gtk_spin_button_new (GTK_ADJUSTMENT (width_adj), 1, 0);
	gtk_widget_show (s->width);
	gtk_table_attach (GTK_TABLE (table), s->width, 1, 2, 0, 1,
			(GtkAttachOptions) (GTK_EXPAND),
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			0, 2);

	/* Height */
	label_height = gtk_label_new (_("Max height"));
	gtk_widget_show (label_height);
	gtk_table_attach (GTK_TABLE (table), label_height, 0, 1, 1, 2,
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		       	2, 0);
	gtk_label_set_justify (GTK_LABEL (label_height), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label_height), 1, 0.5);

	height_adj = gtk_adjustment_new (0, 0, 10000, 1, 10, 10);
	s->height = gtk_spin_button_new (GTK_ADJUSTMENT (height_adj), 1, 0);
	gtk_widget_show (s->height);
	gtk_table_attach (GTK_TABLE (table), s->height, 1, 2, 1, 2,
			(GtkAttachOptions) (GTK_EXPAND),
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		       	0, 2);

	/* Check button */
	s->max_is_abs = gtk_check_button_new_with_mnemonic (
			_("Max is limit"));
	gtk_widget_show (s->max_is_abs);
	gtk_table_attach (GTK_TABLE (table), s->max_is_abs, 0, 2, 2, 3,
			(GtkAttachOptions) (GTK_FILL),
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		       	2, 0);

	/* Format selection */
	label_format = gtk_label_new (_("Format"));
	gtk_widget_show (label_format);
	gtk_table_attach (GTK_TABLE (table), label_format, 0, 1, 3, 4,
			(GtkAttachOptions) (GTK_FILL),
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		       	0, 0);
	gtk_label_set_justify (GTK_LABEL (label_format), GTK_JUSTIFY_RIGHT);
	gtk_misc_set_alignment (GTK_MISC (label_format), 1, 0.5);

	s->format = gtk_option_menu_new ();
	gtk_widget_show (s->format);
	gtk_table_attach (GTK_TABLE (table), s->format, 1, 2, 3, 4,
			(GtkAttachOptions) (0),
			(GtkAttachOptions) (GTK_EXPAND),
		       	0, 0);
	gtk_container_set_border_width (GTK_CONTAINER (s->format), 4);

	menu = gtk_menu_new ();

	/* Add items. Order of items should be the same as in
	 * 'enum img_formats' defined in config_dialog.h
	 */
	item = gtk_radio_menu_item_new_with_mnemonic (
			group, _("PNG"));
	group = gtk_radio_menu_item_get_group(
			GTK_RADIO_MENU_ITEM (item));
	gtk_widget_show (item);
	gtk_container_add (GTK_CONTAINER (menu), item);
	s->format_items[F_PNG] = item;

	item = gtk_radio_menu_item_new_with_mnemonic (
			group, _("JPEG"));
	group = gtk_radio_menu_item_get_group(
			GTK_RADIO_MENU_ITEM (item));
	gtk_widget_show (item);
	gtk_container_add (GTK_CONTAINER (menu), item);
	s->format_items[F_JPEG] = item;

	item = gtk_radio_menu_item_new_with_mnemonic (
			group, _("GIF"));
	gtk_widget_show (item);
	gtk_container_add (GTK_CONTAINER (menu), item);
	s->format_items[F_GIF] = item;
	
	gtk_option_menu_set_menu (GTK_OPTION_MENU (s->format), menu);


	return frame;
}

GtkWidget * create_options_window(struct config_dialog * s)
{
	GtkWidget * vbox;
	GtkWidget * hbox;
	GtkWidget * icon;
	GtkWidget * notebook;

	GtkWidget * table_global;
	GtkWidget * label_global;
	GtkWidget * url_prefix_label;
	GtkWidget * thumb_prefix_label;
	GtkWidget * upd_int_label;
	GtkWidget * upd_int_unit_label;
	GtkWidget * upd_int_help_label;
	GtkObject * upd_int_adj;

	GtkWidget * label_upload;
	GtkWidget * vbox_upload;
	GtkWidget * hbox_upload;
	GtkWidget * label_proto;
	GtkWidget * menu;
	GtkWidget * item;
	GSList * group = NULL;
	GtkWidget * table_upload;
	GtkWidget * host_label;
	GtkWidget * path_label;
	GtkWidget * user_label;

	GtkWidget * label_image;
	GtkWidget * hbox_image;

	GtkWidget * button_box;
	GtkWidget * cancel;
	GtkWidget * save;
	GtkWidget * info_label;
	
	s->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (s->window), _("Edit preferences"));

	vbox = gtk_vbox_new (FALSE, 5);
	gtk_widget_show (vbox);
	gtk_container_add (GTK_CONTAINER (s->window), vbox);

	hbox = gtk_hbox_new (FALSE, 5);
	gtk_widget_show (hbox);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 5);

	icon = gtk_image_new_from_stock (
			GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_SMALL_TOOLBAR);
	gtk_widget_show (icon);
	gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, FALSE, 4);
	
	info_label = gtk_label_new(_("Options marked with "
			       "<b>bold face</b> are required."));
	gtk_widget_show (info_label);
	gtk_label_set_use_markup (GTK_LABEL(info_label), TRUE);
	gtk_label_set_line_wrap (GTK_LABEL(info_label), TRUE);
	gtk_box_pack_start (GTK_BOX (hbox), info_label, TRUE, TRUE, 4);

	notebook = gtk_notebook_new ();
	gtk_widget_show (notebook);
	gtk_box_pack_start (GTK_BOX (vbox), notebook, TRUE, TRUE, 4);
	
	/* Tab: Global options */
	table_global = gtk_table_new (2, 2, FALSE);
	gtk_widget_show (table_global);
	gtk_container_set_border_width (GTK_CONTAINER (table_global), 5);
	
	label_global = gtk_label_new (_("General"));
	gtk_widget_show (label_global);
	gtk_label_set_justify (GTK_LABEL (label_global), GTK_JUSTIFY_LEFT);
	
	gtk_notebook_append_page(GTK_NOTEBOOK (notebook),
			table_global, label_global);

	/* URL prefix */
	url_prefix_label = gtk_label_new (_(config_name[C_URL_PREFIX]));
	gtk_widget_show (url_prefix_label);
	gtk_label_set_use_markup (GTK_LABEL (url_prefix_label), TRUE);
	gtk_label_set_justify (GTK_LABEL (url_prefix_label), GTK_JUSTIFY_RIGHT);
	gtk_misc_set_alignment (GTK_MISC (url_prefix_label), 1, 0.5);
	gtk_table_attach (GTK_TABLE (table_global),
			url_prefix_label,
			0, 1, 0, 1,
			(GtkAttachOptions) (GTK_FILL),
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		       	0, 0);

	s->url_prefix_entry = gtk_entry_new ();
	gtk_widget_show (s->url_prefix_entry);
	gtk_table_attach (GTK_TABLE (table_global),
			s->url_prefix_entry,
			1, 3, 0, 1,
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			5, 5);
	  

	/* Thumbnail prefix */
	thumb_prefix_label = gtk_label_new (_(config_name[C_THUMB_PREFIX]));
	gtk_widget_show (thumb_prefix_label);
	gtk_label_set_use_markup (GTK_LABEL (thumb_prefix_label), TRUE);
	gtk_label_set_justify (GTK_LABEL (thumb_prefix_label),
			GTK_JUSTIFY_RIGHT);
	gtk_misc_set_alignment (GTK_MISC (thumb_prefix_label), 1, 0.5);
	gtk_table_attach (GTK_TABLE (table_global),
			thumb_prefix_label,
		       	0, 1, 1, 2,
			(GtkAttachOptions) (GTK_FILL),
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		       	0, 0);

	s->thumb_prefix_entry = gtk_entry_new ();
	gtk_widget_show (s->thumb_prefix_entry);
	gtk_table_attach (GTK_TABLE (table_global), s->thumb_prefix_entry,
			1, 3, 1, 2,
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		       	5, 5);

	/* Preview update interval */
	upd_int_label = gtk_label_new (_("Update preview each"));
	gtk_widget_show (upd_int_label);
	gtk_label_set_use_markup (GTK_LABEL (upd_int_label), TRUE);
	gtk_label_set_justify (GTK_LABEL (upd_int_label), GTK_JUSTIFY_RIGHT);
	gtk_misc_set_alignment (GTK_MISC (upd_int_label), 1, 0.5);
	
	gtk_table_attach (GTK_TABLE (table_global),
			upd_int_label,
		       	0, 1, 2, 3,
			(GtkAttachOptions) (GTK_FILL),
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		       	0, 0);
	gtk_label_set_justify (GTK_LABEL (upd_int_label), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (upd_int_label), 1, 0.5);

	upd_int_adj = gtk_adjustment_new (1, 0, 10, 0.01, 0.1, 0.1);
	s->preview_upd_int = gtk_spin_button_new (
			GTK_ADJUSTMENT (upd_int_adj), 0.1, 1);
	gtk_widget_show (s->preview_upd_int);
	gtk_table_attach (GTK_TABLE (table_global),
			s->preview_upd_int,
			1, 2, 2, 3,
			(GtkAttachOptions) (GTK_FILL),
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		       	5, 5);

	upd_int_unit_label = gtk_label_new (_("second(s)"));
	gtk_widget_show (upd_int_unit_label);
	gtk_table_attach (GTK_TABLE (table_global),
			upd_int_unit_label,
		       	2, 3, 2, 3,
			(GtkAttachOptions) (GTK_FILL),
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		       	0, 0);

	upd_int_help_label = gtk_label_new (_("<small>Set to 0 "
			"to disable instant update of preview</small>"));
	gtk_widget_show (upd_int_help_label);
	gtk_label_set_use_markup (GTK_LABEL (upd_int_help_label), TRUE);
	gtk_label_set_justify (GTK_LABEL (upd_int_help_label),
			GTK_JUSTIFY_CENTER);
	gtk_misc_set_alignment (GTK_MISC (upd_int_help_label), 0.5, 0.5);
	gtk_table_attach (GTK_TABLE (table_global),
			upd_int_help_label,
		       	0, 3, 3, 4,
			(GtkAttachOptions) (GTK_FILL),
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		       	0, 0);
			       
	/* Tab: Upload options */

	vbox_upload = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox_upload);
	gtk_container_set_border_width (GTK_CONTAINER (vbox_upload), 5);
	
	label_upload = gtk_label_new (_("Upload"));
	gtk_widget_show (label_upload);
	gtk_label_set_justify (GTK_LABEL (label_upload), GTK_JUSTIFY_LEFT);

	gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
			vbox_upload, label_upload);

	/* Upload protocol selection */
	hbox_upload = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox_upload);
	gtk_box_pack_start (GTK_BOX (vbox_upload), hbox_upload,
			TRUE, FALSE, 0);

	label_proto = gtk_label_new (_(config_name[C_UPLOAD_PROTO]));
	gtk_widget_show (label_proto);
	gtk_box_pack_start (GTK_BOX (hbox_upload), label_proto,
		       	TRUE, TRUE, 0);
	gtk_label_set_use_markup (GTK_LABEL (label_proto), TRUE);
	gtk_label_set_justify (GTK_LABEL (label_proto), GTK_JUSTIFY_RIGHT);
	gtk_misc_set_alignment (GTK_MISC (label_proto), 1, 0.5);
	gtk_misc_set_padding (GTK_MISC (label_proto), 5, 0);

	s->upload_proto = gtk_option_menu_new ();
	gtk_widget_show (s->upload_proto);
	gtk_box_pack_start (GTK_BOX (hbox_upload), s->upload_proto,
			TRUE, FALSE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (s->upload_proto), 4);
	

	menu = gtk_menu_new ();

	/* Upload proto option menu. Order of items should be the same
	 * as in 'enum upload_protos' defined in config_dialog.h
	 */
	item = gtk_radio_menu_item_new_with_mnemonic (
			group, _("ftp"));
	group = gtk_radio_menu_item_get_group (
			GTK_RADIO_MENU_ITEM (item));
	gtk_widget_show (item);
	gtk_container_add (GTK_CONTAINER (menu), item);
	s->upload_proto_items[UP_FTP] = item;
	
	item = gtk_radio_menu_item_new_with_mnemonic (
			group, _("scp"));
	gtk_widget_show (item);
	gtk_container_add (GTK_CONTAINER (menu), item);
	s->upload_proto_items[UP_SCP] = item;

	gtk_option_menu_set_menu (GTK_OPTION_MENU (s->upload_proto), menu);

	/* Upload options */
	
	table_upload = gtk_table_new (4, 2, FALSE);
	gtk_widget_show (table_upload);
	gtk_box_pack_start (GTK_BOX (vbox_upload), table_upload,
			TRUE, TRUE, 0);
	
	/* Host */
	host_label = gtk_label_new (_(config_name[C_UPLOAD_HOST]));
	gtk_widget_show (host_label);
	gtk_label_set_use_markup (GTK_LABEL (host_label), TRUE);
	gtk_label_set_justify (GTK_LABEL (host_label), GTK_JUSTIFY_RIGHT);
	gtk_misc_set_alignment (GTK_MISC (host_label), 1, 0.5);
	gtk_table_attach (GTK_TABLE (table_upload), host_label,
			0, 1, 0, 1,
			(GtkAttachOptions) (GTK_FILL),
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		       	0, 0);

	s->upload_host = gtk_entry_new ();
	gtk_widget_show (s->upload_host);
	gtk_table_attach (GTK_TABLE (table_upload), s->upload_host,
			1, 2, 0, 1,
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		       	5, 2);
	
	/* Path */
	path_label = gtk_label_new (_(config_name[C_UPLOAD_PATH]));
	gtk_widget_show (path_label);
	gtk_label_set_use_markup (GTK_LABEL (path_label), TRUE);
	gtk_label_set_justify (GTK_LABEL (path_label), GTK_JUSTIFY_RIGHT);
	gtk_misc_set_alignment (GTK_MISC (path_label), 1, 0.5);
	gtk_table_attach (GTK_TABLE (table_upload), path_label,
			0, 1, 1, 2,
			(GtkAttachOptions) (GTK_FILL),
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		       	0, 0);

	s->upload_path = gtk_entry_new ();
	gtk_widget_show (s->upload_path);
	gtk_table_attach (GTK_TABLE (table_upload), s->upload_path,
			1, 2, 1, 2,
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		       	5, 2);
	
	/* User */
	user_label = gtk_label_new (_(config_name[C_UPLOAD_USER]));
	gtk_widget_show (user_label);
	gtk_label_set_use_markup (GTK_LABEL (user_label), TRUE);
	gtk_label_set_justify (GTK_LABEL (user_label), GTK_JUSTIFY_RIGHT);
	gtk_misc_set_alignment (GTK_MISC (user_label), 1, 0.5);
	gtk_table_attach (GTK_TABLE (table_upload), user_label,
			0, 1, 2, 3,
			(GtkAttachOptions) (GTK_FILL),
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			0, 0);

	s->upload_user = gtk_entry_new ();
	gtk_widget_show (s->upload_user);
	gtk_table_attach (GTK_TABLE (table_upload), s->upload_user,
			1, 2, 2, 3,
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			5, 2);
	
	/* Pass */
	s->upload_pass_label = gtk_label_new (_(config_name[C_UPLOAD_PASS]));
	gtk_widget_show (s->upload_pass_label);
	gtk_label_set_use_markup (GTK_LABEL (s->upload_pass_label), TRUE);
	gtk_label_set_justify (GTK_LABEL (s->upload_pass_label),
			GTK_JUSTIFY_RIGHT);
	gtk_misc_set_alignment (GTK_MISC (s->upload_pass_label), 1, 0.5);
	gtk_table_attach (GTK_TABLE (table_upload), s->upload_pass_label,
			0, 1, 3, 4,
			(GtkAttachOptions) (GTK_FILL),
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			0, 0);

	s->upload_pass = gtk_entry_new ();
	gtk_widget_show (s->upload_pass);
	gtk_entry_set_visibility (GTK_ENTRY (s->upload_pass), FALSE);
	gtk_table_attach (GTK_TABLE (table_upload), s->upload_pass,
			1, 2, 3, 4,
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		       	5, 2);

	/* Tab: Image options */

	hbox_image = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox_image);
	
	label_image = gtk_label_new (_("Image"));
	gtk_widget_show (label_image);
	gtk_label_set_justify (GTK_LABEL (label_image), GTK_JUSTIFY_LEFT);

	gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
			hbox_image, label_image);

	/* Image and Thumbnail frames */
	gtk_box_pack_start (GTK_BOX (hbox_image),
			create_image_options (_("Image"), &s->image),
			TRUE, TRUE, 3);

	gtk_box_pack_start (GTK_BOX (hbox_image),
			create_image_options (_("Thumbnail"), &s->thumb),
			TRUE, TRUE, 3);
	
	/* Buttons */
	button_box = gtk_hbox_new (FALSE, 5);
	gtk_widget_show (button_box);
	gtk_box_pack_start (GTK_BOX (vbox), button_box,
		       	TRUE, TRUE, 15);	
	
	cancel = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
	gtk_widget_show (cancel);
	gtk_box_pack_start (GTK_BOX (button_box), cancel,
			TRUE, FALSE, 5);
	
	save = gtk_button_new_from_stock (GTK_STOCK_SAVE);
	gtk_widget_show (save);
	gtk_box_pack_start (GTK_BOX (button_box), save,
			TRUE, FALSE, 5);

	/* Event handlers */
	g_signal_connect ((gpointer) s->upload_proto, "changed",
		G_CALLBACK (upload_proto_callback), (gpointer) s);

	/* FIXME: ask whether to save or discard settings */
	g_signal_connect ((gpointer) s->window, "delete_event",
		G_CALLBACK (gtk_main_quit), NULL);

	g_signal_connect ((gpointer) cancel, "clicked",
		G_CALLBACK (gtk_main_quit), NULL);
	
	g_signal_connect ((gpointer) save, "clicked",
			G_CALLBACK (on_config_save_clicked),
			(gpointer) s);

	return s->window;
}

gboolean show_config_dialog(GtkWindow * parent)
{
	GtkWindow * opt;
	opt = GTK_WINDOW (create_options_window (&cd));
	set_dialog_from_config (&cd, config);

	gtk_window_set_modal (opt, TRUE);
	if (parent)
	{
		gtk_window_set_transient_for (opt, parent);
		gtk_window_set_destroy_with_parent (opt, TRUE);
	}

	gtk_widget_show (GTK_WIDGET (opt));
	cd.saved = FALSE;

	gtk_main();
	gtk_widget_destroy (GTK_WIDGET (opt));
	return cd.saved;
}

gboolean check_config (const char *config[])
{
	GString * errors;
	int i;
	int errcount;

	errcount = 0;
	errors = g_string_sized_new(128);

	for (i = 0; i < C_END; i++)
	{
		if (!config_required[i])
			continue;
		if ((config[i] == NULL) || (config[i][0] == '\0'))
		{
			g_string_append_printf(errors,
					"- %s\n",
					config_name[i]);
			errcount++;
		}
	}

	if (errcount == 0)
		return TRUE;

	show_error(_("Error"),
			_("<big>Some configuration options are required "
				"but they are empty.\nPlease fill in "
				"the following fields:</big>"),
			errors->str);
	return FALSE;
}
