/* $Id$
 * Simple generic error dialog.
 */
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include "interface.h"
#include "support.h"

extern struct mainwin win;

GtkWidget * create_dialog(const char * wt,
		const char * err,
		const char * add)
{
	GtkWidget *dialog;
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *img;
	GtkWidget *errlabel;
	GtkWidget *errtext;
	GtkWidget *action_area;
	GtkAccelGroup *accel_group;

	dialog = gtk_dialog_new_with_buttons (
			wt,
			GTK_WINDOW (win.window),
			GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
			GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT,
			NULL);
	gtk_window_set_position (GTK_WINDOW (dialog),
			GTK_WIN_POS_MOUSE);

	accel_group = gtk_accel_group_new ();
	gtk_widget_add_accelerator (dialog, "close", accel_group,
			GDK_Escape, 0, GTK_ACCEL_VISIBLE);

	vbox = GTK_DIALOG (dialog)->vbox;
	gtk_widget_show (vbox);

	hbox =  gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 5);

	img = gtk_image_new_from_stock ("gtk-dialog-error",
			GTK_ICON_SIZE_DIALOG);
	gtk_widget_show (img);
	gtk_box_pack_start (GTK_BOX (hbox), img, TRUE, TRUE, 0);

	errlabel = gtk_label_new (err);
	gtk_widget_show (errlabel);
	gtk_box_pack_start (GTK_BOX (hbox), errlabel, TRUE, TRUE, 0);
	gtk_label_set_use_markup (GTK_LABEL (errlabel), TRUE);
	gtk_label_set_line_wrap (GTK_LABEL (errlabel), TRUE);
	gtk_label_set_justify (GTK_LABEL (errlabel), GTK_JUSTIFY_CENTER);
	gtk_misc_set_alignment (GTK_MISC (errlabel), 0.5, 0.5);


	errtext = gtk_label_new (add);
	gtk_widget_show (errtext);
	gtk_box_pack_start (GTK_BOX (vbox), errtext, TRUE, TRUE, 10);
	gtk_label_set_use_markup (GTK_LABEL (errtext), TRUE);
	gtk_label_set_justify (GTK_LABEL (errtext), GTK_JUSTIFY_LEFT);

	action_area = GTK_DIALOG (dialog)->action_area;
	gtk_widget_show (action_area);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (action_area),
			GTK_BUTTONBOX_END);
	
	gtk_window_add_accel_group (GTK_WINDOW (dialog), accel_group);

	return dialog;
}

void show_error(const char * wintitle, const char * err,
		const char * output)
{
	GtkWidget * dialog;

	dialog = create_dialog (wintitle, err, output);
	gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_destroy (dialog);
}

void show_error_tt(const char * wintitle, const char * err,
		const char * output, int len)
{
	char output_prefix[] = "<tt><small>";
	char output_suffix[] = "</small></tt>";
	char * s = NULL;
	
	if (output)
	{
		int slen = len;
		if (len == -1)
			slen = strlen(output);
		s = (char *) malloc (sizeof(output_prefix) - 1
				+ slen + sizeof(output_suffix));
		strncpy(s, output_prefix, sizeof(output_prefix) - 1);
		strncpy(s + sizeof(output_prefix) - 1, output, slen);
		strncpy(s + sizeof(output_prefix) - 1 + slen,
				output_suffix, sizeof(output_suffix));
	}
	show_error(wintitle, err, s);
	if (s != NULL) 
		free(s);
}
