#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>
#include "support.h"
#include "imgsize.h"
#include "config_file.h"
#include "htmlgen.h"
#include "interface.h"

/*
 * <a href="[url_prefix][img_name]" target="_blank"><img
 * src="[url_prefix][tn_name]"
 * width="[img_width]" height="[img_height]"
 * alt="[img_alt]" border="0" /></a>
 */

extern struct mainwin win;
extern const char * config[C_END];
extern struct imgsize is;
extern struct imgsize ts;

GString * generate_html(const struct html_data * hd)
{
	struct imgsize * iss;
	GString *s;
	
	s = g_string_sized_new(128);
	iss = (hd->is_thumbnailed)? &ts: &is;
	
	if (hd->is_thumbnailed)
	{
		g_string_append_printf (s,
			"<A HREF=\"%s%s\"%s>",
			config[C_URL_PREFIX],
			hd->filename_utf8,
			(hd->is_in_new_window)? " TARGET=\"_blank\"": "");
	}
	g_string_append_printf (s,
		"<IMG SRC=\"%s%s%s\" WIDTH=\"%d\" HEIGHT=\"%d\" "
		"ALT=\"%s\" TITLE=\"%s\" BORDER=\"%d\"/>",
		config[C_URL_PREFIX],
		(hd->is_thumbnailed)?  config[C_THUMB_PREFIX]: "",
		hd->filename_utf8,
		iss->scaled_size_x,
		iss->scaled_size_y,
		(hd->alt)? hd->alt: "",
		(hd->title)? hd->title: "",
		hd->border);
	if (hd->is_thumbnailed)
	{
		g_string_append (s, "</A>");
	}

	return s;
}

GString * get_html_code (const struct html_data * hd)
{
	GString * s, * ns;
	GtkWidget * dialog;
	GtkWidget * vbox;
	GtkWidget * view;
	GtkWidget * label;
	GtkTextBuffer *buffer;
	GtkTextIter start, end;
	gint resp;
	
	s = generate_html(hd);

	/* Show edit dialog */
	dialog = gtk_dialog_new_with_buttons (
			_("HTML code"),
			GTK_WINDOW (win.window),
			GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_OK, GTK_RESPONSE_OK,
			NULL);

	vbox = GTK_DIALOG (dialog)->vbox;
	
	label = gtk_label_new (_("<big>Resulting HTML code is below.\n"
				"You can edit it here if needed.</big>"));
	gtk_widget_show (label);
	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_CENTER);
	gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0.5);
	gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 5);
	
	view = gtk_text_view_new ();
	gtk_widget_show (view);

	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
	gtk_text_buffer_set_text (buffer, s->str, s->len);

	gtk_text_view_set_editable (GTK_TEXT_VIEW (view), TRUE);
	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (view), GTK_WRAP_WORD);
	
	gtk_box_pack_start (GTK_BOX (vbox), view, TRUE, TRUE, 5);

	/* Run dialog */
	resp = gtk_dialog_run (GTK_DIALOG (dialog));
	
	if (resp == GTK_RESPONSE_OK)
	{
		gtk_text_buffer_get_bounds(buffer, &start, &end);
		ns = g_string_new (
			gtk_text_buffer_get_text (buffer, &start, &end, TRUE));
	}
	else
	{
		ns = NULL;
	}
		
	gtk_widget_destroy (dialog);
	g_string_free (s, TRUE);

	return ns;
}
