/* img-ops.c
 *
 * Copyright (C) 2003 by Kir Kolyshkin <kir@sacred.ru>
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <glib.h>
#include "support.h"
#include "dialog.h"

/* FIXME: we are lame and use global variables */
static int img_width = -1;
static int img_height = -1;

void show_resize_error(const char * str, int strlen)
{
	show_error_tt(_("Error resizing image"),
			_("<big><b>Error:</b> image resizing failed</big>"),
			str, strlen);
}

const char * get_image_info(const char * filename)
{
	/* format is: TYPE WIDTHxHEIGHT SIZE */
	const char imi[] = "identify -format '%m %wx%h %b' ";
	char * cmd;
	FILE * stream;
	static char buf[100];

	cmd = (char *)malloc(sizeof(imi) + strlen(filename));
	cmd[0] = '\0';
	strcpy(cmd, imi);
	strcat(cmd, filename);
	fprintf(stderr, "+ popen(\"%s\")\n", cmd);

	stream = popen(cmd, "r");
	if (stream == NULL)
		return NULL;
	fgets(buf, sizeof(buf), stream);
	pclose(stream);
	free(cmd);
	/* remove '\n' */
	buf[strlen(buf) - 1] = '\0';
	sscanf(buf, "%*s %dx%d %*s", &img_width, &img_height);

	return buf;
}

int get_image_size(int *width, int *height)
{
	if ((img_width == -1) || (img_height == -1))
		return -1;
	
	*width  = img_width;
	*height = img_height;

	return 0;
}

int image_resize(const char * filename, int new_width, int new_height,
		const char * new_filename)
{
	GString * cmd;
	GString * output;
	char buf[BUFSIZ];
	FILE * stream;
	int ret;

	cmd = g_string_sized_new (128);
	output = g_string_sized_new (BUFSIZ);

	g_string_printf (cmd, "convert -geometry %dx%d %s %s 2>&1",
			new_width, new_height,
			filename, new_filename);

	fprintf(stderr, "+ popen(\"%s\")\n", cmd->str);
	
	stream = popen(cmd->str, "r");
	if (stream == NULL)
	{
		ret = -1;
		g_string_printf (output, "popen(\"%s\") failed: %s\n",
				cmd->str, strerror(errno));
		show_resize_error(output->str, output->len);
		goto out;
	}
	
	g_string_printf (output, "+ \"%s\"\n", cmd->str);
	while (fgets(buf, BUFSIZ, stream) != NULL)
		g_string_append (output, buf);
	ret = pclose(stream);
	if (ret != 0)
	{
		show_resize_error(output->str, output->len);
	}
out:
	g_string_free (cmd, TRUE);
	g_string_free (output, TRUE);
	return ret;
}

