#include "imgsize.h"
#include "support.h"
#include "htmlgen.h"

extern struct html_data hd;

void slider_callback(GtkRange * range, gpointer _is);
void spin_x_callback(GtkSpinButton *spinbutton, gpointer _is);
void spin_y_callback(GtkSpinButton *spinbutton, gpointer _is);

int block_all_callbacks(struct imgsize * is)
{
	return
		g_signal_handlers_block_by_func(
			is->spin_x, spin_x_callback, is) +
		g_signal_handlers_block_by_func(
			is->spin_y, spin_y_callback, is) +
		g_signal_handlers_block_by_func(
			is->slider, slider_callback, is);
}

int unblock_all_callbacks(struct imgsize * is)
{
	return
		g_signal_handlers_unblock_by_func(
			is->spin_x, spin_x_callback, is) +
		g_signal_handlers_unblock_by_func(
			is->spin_y, spin_y_callback, is) +
		g_signal_handlers_unblock_by_func(
			is->slider, slider_callback, is);
}

void set_slider(struct imgsize * is)
{
	gtk_range_set_value(GTK_RANGE(is->slider), is->scale * 100);
}

void set_spin_x(struct imgsize * is)
{
	gtk_spin_button_set_value(
		GTK_SPIN_BUTTON(is->spin_x), (gdouble)is->scaled_size_x);
}

void set_spin_y(struct imgsize * is)
{
	gtk_spin_button_set_value(
		GTK_SPIN_BUTTON(is->spin_y), (gdouble)is->scaled_size_y);
}

void rescale(struct imgsize * is, gdouble new_scale)
{
	is->scaled_size_x = is->size_x * new_scale;
	is->scaled_size_y = is->size_y * new_scale;
	is->scale = new_scale;
	block_all_callbacks(is);
	set_slider(is);
	set_spin_x(is);
	set_spin_y(is);
	unblock_all_callbacks(is);
}

/* Call this upon loading an image */
void set_image_size(struct imgsize * is, int x, int y)
{
	GtkAdjustment* adj;
	is->size_x = x;
	is->size_y = y;
	
	/* Set max values for spinbuttons */
	adj = gtk_spin_button_get_adjustment(is->spin_x);
	adj->upper = (gdouble) x;

	adj = gtk_spin_button_get_adjustment(is->spin_y);
	adj->upper = (gdouble) y;
	
	/* redraw all */
	rescale(is, 1);
}

GtkWidget * create_image_resizer(struct imgsize * is, GtkWidget * container)
{
	GtkWidget *vbox;
	GtkWidget *hbox1;
	GtkWidget *hscale1;
	GtkWidget *label1;
	GtkWidget *hbox2;
	GtkWidget *label4;
	GtkWidget *label2;
	GtkObject *spinbutton1_adj;
	GtkWidget *spinbutton1;
	GtkWidget *label3;
	GtkObject *spinbutton2_adj;
	GtkWidget *spinbutton2;
	GtkWidget *label5;
	
	vbox = gtk_vbox_new (TRUE, 0);
	gtk_widget_show (vbox);
	hbox1 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (vbox), hbox1, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (hbox1), 5);

	hscale1 = gtk_hscale_new(GTK_ADJUSTMENT(gtk_adjustment_new(
					100, 0, 100, 1, 5, 0)));
	gtk_widget_show (hscale1);
	gtk_box_pack_start (GTK_BOX (hbox1), hscale1, TRUE, TRUE, 0);
	gtk_scale_set_value_pos (GTK_SCALE (hscale1), GTK_POS_RIGHT);
	gtk_scale_set_digits (GTK_SCALE (hscale1), 2);
	gtk_range_set_update_policy (GTK_RANGE (hscale1), GTK_UPDATE_DELAYED);

	label1 = gtk_label_new ("%");
	gtk_widget_show (label1);
	gtk_box_pack_end (GTK_BOX (hbox1), label1, FALSE, FALSE, 0);
	gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_LEFT);

	hbox2 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox2);
	gtk_box_pack_start (GTK_BOX (vbox), hbox2, FALSE, FALSE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (hbox2), 5);

	label4 = gtk_label_new ("");
	gtk_widget_show (label4);
	gtk_box_pack_start (GTK_BOX (hbox2), label4, TRUE, TRUE, 0);

	label2 = gtk_label_new (_("Width:"));
	gtk_widget_show (label2);
	gtk_box_pack_start (GTK_BOX (hbox2), label2, FALSE, FALSE, 2);
	gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_LEFT);
//	gtk_misc_set_padding (GTK_MISC (label2), 0, 10);

	spinbutton1_adj = gtk_adjustment_new (0, 0, 100, 1, 10, 10);
	spinbutton1 = gtk_spin_button_new (
			GTK_ADJUSTMENT (spinbutton1_adj), 1, 0);
	gtk_widget_show (spinbutton1);
	gtk_box_pack_start (GTK_BOX (hbox2), spinbutton1, FALSE, FALSE, 4);
	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton1), TRUE);

	label3 = gtk_label_new (_("Height:"));
	gtk_widget_show (label3);
	gtk_box_pack_start (GTK_BOX (hbox2), label3, FALSE, FALSE, 6);
	gtk_label_set_justify (GTK_LABEL (label3), GTK_JUSTIFY_LEFT);
//	gtk_misc_set_padding (GTK_MISC (label3), 0, 10);
	
	spinbutton2_adj = gtk_adjustment_new (0, 0, 100, 1, 10, 10);
	spinbutton2 = gtk_spin_button_new (
			GTK_ADJUSTMENT (spinbutton2_adj), 1, 0);
	gtk_widget_show (spinbutton2);
	gtk_box_pack_start (GTK_BOX (hbox2), spinbutton2, FALSE, FALSE, 0);

	label5 = gtk_label_new ("");
	gtk_widget_show (label5);
	gtk_box_pack_start (GTK_BOX (hbox2), label5, TRUE, TRUE, 5);

	is->preview = gtk_check_button_new_with_label (_("Preview"));
	gtk_widget_show (is->preview);
	gtk_box_pack_start (GTK_BOX (hbox2), is->preview, FALSE, FALSE, 0);

	g_signal_connect ((gpointer) hscale1, "value_changed",
			G_CALLBACK (slider_callback), (gpointer) is);
	g_signal_connect ((gpointer) spinbutton1, "value_changed",
			G_CALLBACK (spin_x_callback), (gpointer) is);
	g_signal_connect ((gpointer) spinbutton2, "value_changed",
			G_CALLBACK (spin_y_callback), (gpointer) is);

	if (container == NULL)
		is->container = vbox;
	else
		is->container = container;
	is->slider = GTK_RANGE(hscale1);
	is->spin_x = GTK_SPIN_BUTTON(spinbutton1);
	is->spin_y = GTK_SPIN_BUTTON(spinbutton2);

	gtk_widget_set_sensitive(is->container, FALSE);

	return vbox;
}

void spin_x_callback(GtkSpinButton *spinbutton, gpointer _is)
{
	struct imgsize * is = (struct imgsize *)_is;
       
	is->scaled_size_x = gtk_spin_button_get_value_as_int(
		GTK_SPIN_BUTTON(spinbutton));
	is->scale = (double)is->scaled_size_x / (double)is->size_x;
	is->scaled_size_y = is->size_y * is->scale;
	block_all_callbacks(is);
	set_slider(is);
	set_spin_y(is);
	unblock_all_callbacks(is);
}

void spin_y_callback(GtkSpinButton *spinbutton, gpointer _is)
{
	struct imgsize * is = (struct imgsize *)_is;

	is->scaled_size_y = gtk_spin_button_get_value_as_int(
		GTK_SPIN_BUTTON(spinbutton));
	is->scale = (double)is->scaled_size_y / (double)is->size_y; 
	is->scaled_size_x  = is->size_x * is->scale;
	block_all_callbacks(is);
	set_slider(is);
	set_spin_x(is);
	unblock_all_callbacks(is);
}

void slider_callback(GtkRange * range, gpointer _is)
{
	struct imgsize * is = (struct imgsize *)_is;

	is->scale = gtk_range_get_value(GTK_RANGE(range)) / 100.;
	is->scaled_size_x  = is->size_x * is->scale;
	is->scaled_size_y = is->size_y * is->scale;
	block_all_callbacks(is);
	set_spin_x(is);
	set_spin_y(is);
	unblock_all_callbacks(is);
}

/* Rescale thumbnal size to be not large than scaled image size */
void tscale_callback(GtkRange * range, gpointer _ts)
{
	struct imgsize * ts = (struct imgsize *)_ts;
	gdouble new_scale = gtk_range_get_value(GTK_RANGE(range));
	if (new_scale < gtk_range_get_value(GTK_RANGE(ts->slider)))
		rescale(ts, new_scale / 100.);
}
