#include <gtk/gtk.h>
#include <stdio.h>
#include <assert.h>
#include <string.h> /* for memset() */
#include <stdlib.h> /* for atoi() */
#include "support.h"
#include "preview.h"
#include "filename.h"
#include "config_file.h"
#include "interface.h"
#include "dialog.h"

struct preview_window pv_orig;
struct preview_window pv_image;
struct preview_window pv_thumb;
extern char * config[];
extern struct mainwin win;

void destroy_preview_all (void);

void button_preview_set_sensitive_all (gboolean active)
{
	gtk_widget_set_sensitive (pv_orig.button, active);
	gtk_widget_set_sensitive (pv_image.button, active);
	gtk_widget_set_sensitive (pv_thumb.button, active);
}

void close_preview_window (GtkWidget * but, gpointer user_data)
{
	gtk_toggle_button_set_active (
			GTK_TOGGLE_BUTTON (but), FALSE);
}

void button_preview_uncheck_all (void)
{
	gtk_toggle_button_set_active (
			GTK_TOGGLE_BUTTON (pv_orig.button), FALSE);
	gtk_toggle_button_set_active (
			GTK_TOGGLE_BUTTON (pv_image.button), FALSE);
	gtk_toggle_button_set_active (
			GTK_TOGGLE_BUTTON (pv_thumb.button), FALSE);
}

void init_preview(struct imgsize * is, struct imgsize * ts)
{
	memset(&pv_orig, 0, sizeof(pv_orig));
	memset(&pv_image, 0, sizeof(pv_image));
	memset(&pv_thumb, 0, sizeof(pv_thumb));
			
	pv_image.is = is;
	pv_thumb.is = ts;

	pv_orig.title = N_("Original image preview");
	pv_image.title = N_("Scaled image preview");
	pv_thumb.title = N_("Thumbnail preview");
}

int set_preview_file (const char * filename)
{
	GError * gerror = NULL;
	GdkPixbuf * pixbuf;
	
	pixbuf = gdk_pixbuf_new_from_file (filename, &gerror);
	if (pixbuf == NULL)
	{
/*		GString * s = g_string_sized_new (128);
		
		button_preview_set_sensitive_all(FALSE);
		g_string_printf (s, N_("<big><b>Error:</b> can't load "
				"preview from %s:\n</big><tt>%s</tt>\n"
				"<big>Preview will be unavailable</big>"),
				filename, gerror->message);
		show_error(_("Error loading preview"),
				_(s->str), NULL);
*/
		return -1;
	}
	button_preview_set_sensitive_all(TRUE);
	destroy_preview_all();
	pv_orig.orig_pb = pv_orig.pb = pixbuf;
	pv_image.orig_pb = pv_thumb.orig_pb = pixbuf;
	return 0;
}

void refresh_preview(struct preview_window * p)
{
	gtk_image_set_from_pixbuf (p->image, p->pb);
/*	gtk_widget_queue_draw (p->image); */
}

void create_preview_window (struct preview_window * p)
{
	GtkWidget * vbox;

	p->w = GTK_WINDOW (gtk_window_new (GTK_WINDOW_TOPLEVEL));
	/* Window is resizable by program, not by user */
	gtk_window_set_resizable (p->w, FALSE);
	gtk_window_set_title (p->w, _(p->title));

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox);
	gtk_container_add (GTK_CONTAINER (p->w), vbox);

	assert(p->pb != NULL);
	p->image = GTK_IMAGE (gtk_image_new_from_pixbuf (p->pb));
	gtk_widget_show (GTK_WIDGET (p->image));
	
	gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET (p->image),
			FALSE, FALSE, 0);

/*
	btn = gtk_button_new_from_stock ("gtk-close");
	gtk_widget_show (btn);
	gtk_box_pack_start (GTK_BOX (vbox), btn, FALSE, FALSE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (btn), 10);
	
	g_signal_connect_swapped ((gpointer) btn, "clicked",
			G_CALLBACK (gtk_widget_destroy), p->button);
*/
	assert(p->button != NULL);
	g_signal_connect_swapped ((gpointer) p->w, "delete_event",
			G_CALLBACK (close_preview_window), p->button);

	gtk_widget_show (GTK_WIDGET (p->w));
	/* Set focus to main window */
	gtk_window_present (GTK_WINDOW (win.window));
}

/* Returns:
 *   0 - resizing not needed 
 *   1 - resized
 *  -1 - resizing error
 */
int resize_if_needed (struct preview_window * p)
{
	if (p->is == NULL)
		return 0;

	assert (p->pb != p->orig_pb);

	if ((p->pb != NULL) &&
			((p->x == p->is->scaled_size_x) ||
			(p->y == p->is->scaled_size_y)))
		return 0;
	
	if (p->pb != NULL)
		g_object_unref(p->pb);
	
	p->pb = gdk_pixbuf_scale_simple (
			p->orig_pb,
			p->is->scaled_size_x,
			p->is->scaled_size_y,
			GDK_INTERP_NEAREST);
	if (p->pb == NULL)
		return -1;
	p->x = p->is->scaled_size_x;
	p->y = p->is->scaled_size_y;
	return 1;
}

gboolean update_preview (gpointer user_data)
{
	struct preview_window * p = (struct preview_window *) user_data;
	assert (p->w != NULL);

	if (resize_if_needed(p) == 1)
		/* resized */
		refresh_preview (p);
	return TRUE;
}

void reg_update_preview(struct preview_window * p)
{
	guint32 interval;

	if (p->is == NULL)
		/* original image - not going to update it*/
		return;
	
	if (p->teh != 0)
		return; /* already registered */
	
	if (config[C_PREVIEW_UPD_INT] == NULL)
		return;
	interval = atoi(config[C_PREVIEW_UPD_INT]);
	if (interval == 0)
		return;
	
	p->teh = gtk_timeout_add (interval,
		update_preview,	(gpointer)p);
}

void unreg_update_preview(struct preview_window * p)
{
	if (p->teh == 0)
		return;
	
	gtk_timeout_remove (p->teh);
	p->teh = 0;
}

void destroy_preview_window (struct preview_window * p)
{
	unreg_update_preview (p);
	if (p->w != NULL)
	{
		gtk_widget_destroy (GTK_WIDGET (p->w));
		p->w = NULL;
	}
	/* free pixbuf */
	if ((p->pb != NULL) && (p->pb != p->orig_pb))
	{
		g_object_unref (p->pb);
		p->pb = NULL;
	}
}

void destroy_preview_all (void)
{
/*
	destroy_preview_window (&pv_orig);
	destroy_preview_window (&pv_image);
	destroy_preview_window (&pv_thumb);
*/
	button_preview_uncheck_all();
	/* Free original image pixbuf */
	if (pv_orig.orig_pb != NULL)
	{
		g_object_unref (pv_orig.orig_pb);
		pv_orig.orig_pb = NULL;
	}
}

void preview_callback (GtkToggleButton * button, gpointer user_data)
{
	gboolean active;
	struct preview_window * p = (struct preview_window *) user_data;

	active = gtk_toggle_button_get_active (button);
	if (active == TRUE)
	{ /* Preview is ON */
		if (p->w == NULL)
		{ /* no window, so create one */
			resize_if_needed(p);
			create_preview_window (p);
			reg_update_preview(p);
		}
/*		else
		{
			if (resize_if_needed(p) == 1)
				refresh_preview (p);
			reg_update_preview(p);
		}
*/
	}
	else
	{ /* Preview is OFF */
		destroy_preview_window(p);
	}
}

void connect_preview_signals(void)
{
	g_signal_connect ((gpointer) pv_orig.button, "toggled",
			G_CALLBACK (preview_callback), &pv_orig);
	g_signal_connect ((gpointer) pv_image.button, "toggled",
			G_CALLBACK (preview_callback), &pv_image);
	g_signal_connect ((gpointer) pv_thumb.button, "toggled",
			G_CALLBACK (preview_callback), &pv_thumb);
}

void connect_preview_buttons(GtkWidget * b1, GtkWidget * b2, GtkWidget * b3)
{
	pv_orig.button = b1;
	pv_image.button = b2;
	pv_thumb.button = b3;
	connect_preview_signals();
}
